/*
 * mcasp_in.c, stammt aus ...\platform\evmAM335x\mcasp.c
 *
 *  Created on: 19 Dec 2013
 *      Author: Manfred
 */

/**
 * \file   mcasp.c
 *
 * \brief  This file contains functions which configure McASP pins
 */

/*
 #include "soc_AM335x.h"
 #include "hw_control_AM335x.h"
 #include "hw_types.h"
 #include "evmAM335x.h"
 #include "hw_cm_per.h"
 */

#include "configEnetEcho.h"

#define MCASP_SEL_MODE            0x04

//******************************************************************************
//**                      INTERNAL MACRO DEFINITIONS
//******************************************************************************
/*
 ** Values which are configurable
 */
/* Slot size to send/receive data */
#define SLOT_SIZE                             (32u)                   // vorher: (16u)
/* Word size to send/receive data. Word size <= Slot size */
#define WORD_SIZE_MASK						  (24u) // bestimmt RMASK-Register (
#define WORD_SIZE                             (32u)	//vorher: (16u)
// bestimmt: AUDIO_BUF_SIZE
//           BYTES_PER_SAMPLE

/* Sampling Rate which will be used by both transmit and receive sections */
//#define SAMPLING_RATE                         (192000u)  	// war (48000u)
// Number of channels, L & R,
#define NUM_I2S_CHANNELS                      (4u)          // vorher: (2u),  offensichtlich immer 2 f. I2S, unabh. v. Anz. d. Serializer!
// - bestimmt Anz. der Serializer (NUM_RX_SERIALIZERS)
// - in McASPPinDirInputSet() bei Bedarf weitere Serializer als Eingang vereinbaren
// - mit McASPSerializerRxSet() bei Bedarf weitere Serializer aktivieren
// - mit McASPPinMcASPSet_in()  bei Bedarf weitere Serializer als McASP-Pin vereinbaren

/* Number of samples to be used per audio buffer */
#define NUM_SAMPLES_PER_AUDIO_BUF             (64u)           // vorher: (2000u)    	// anpassen!!!!
/* Number of buffers used per tx/rx */
#define NUM_BUF                               (3u)          // vorher: (3u), Annzahl der Empf.-Puffer, siehe rxBufPtr[NUM_BUF]
// Pro Durchlauf wird ein Puffer beschrieben
// Number of linked parameter set used per tx/rx
#define NUM_PAR                               NUM_BUF		// vorher: (2u)
// 1 --> nur ein ParRAM (u. damit nur ein Puffer) fr einen Durchlauf , keine verlinkten ParRAMs,
/* Specify where the parameter set starting is */
#define PAR_ID_START                          (70u) // ab diesem ParRAM Sampledaten empfangen
/* Number of samples in loop buffer */
//#define NUM_SAMPLES_LOOP_BUF                  (10u)
// AIC3106 codec address
//#define I2C_SLAVE_CODEC_AIC31                 (0x1Bu)
// McASP Serializer for Receive, Nr. des seriellen McASP-Einganges (3...0)
//vorher: #define MCASP_XSER_RX                         (3u)
#define MCASP_XSER_RX_SDOUT1 (0u)    // HF- u. NF-Signal an Eingang 0, stammt von SDOUT1 des ADC,
#define MCASP_XSER_RX_SDOUT2 (1u)  // ser. Eingang 1 (Signal SDOUT2 des ADC)     !!!!!!!!!!!!!!!!!!!!!!
// McASP Serializer for Transmit
//#define MCASP_XSER_TX                         (2u)
//** Below Macros are calculated based on the above inputs
#define NUM_RX_SERIALIZERS                    ((NUM_I2S_CHANNELS >> 1) + (NUM_I2S_CHANNELS & 0x01))

//#define I2S_SLOTS                             ((1 << NUM_I2S_CHANNELS) - 1)    // 3 fr RTDM ?????????????
#define I2S_SLOTS 3   // Maske fr die zu aktivierenden Time Slots, also slot1 und slot0
// nur 2 Slots, da beide Serializer parallel arbeiten (gleicher Takt)

#define BYTES_PER_SAMPLE  ((WORD_SIZE >> 3) * NUM_I2S_CHANNELS)  // d.h. f. 2 I2S-Signale (4 channels)-->16 Bytes
#define AUDIO_BUF_SIZE    (NUM_SAMPLES_PER_AUDIO_BUF  * BYTES_PER_SAMPLE)  // in BYTES_PER_SAMPLE ist schon NUM_RX_SERIALIZERS bercksichtigt
// auf MCASP0 gendert:
#define RX_DMA_INT_ENABLE                     (EDMA3CC_OPT_TCC_SET \
                                               (EDMA3_CHA_MCASP0_RX) | (1 \
                                               << EDMA3CC_OPT_TCINTEN_SHIFT))
// (EDMA3CC_OPT_TCC_SET (EDMA3_CHA_MCASP0_RX) | (1 << EDMA3CC_OPT_TCINTEN_SHIFT))
//                                 9          |                0x14u
/*
 volatile unsigned int dw0, dw1, dw2, dw3, dw4;
 dw0 = NUM_RX_SERIALIZERS;	// ((NUM_I2S_CHANNELS >> 1) + (NUM_I2S_CHANNELS & 0x01))
 dw1 = I2S_SLOTS;			// ((1 << NUM_I2S_CHANNELS) - 1)
 dw2 = BYTES_PER_SAMPLE;		// ((WORD_SIZE >> 3) * NUM_I2S_CHANNELS)
 dw3 = AUDIO_BUF_SIZE;       // (NUM_SAMPLES_PER_AUDIO_BUF * BYTES_PER_SAMPLE)
 dw4 = NUM_I2S_CHANNELS;
 dw5 = NUM_SAMPLES_PER_AUDIO_BUF;
 *
 */
/** OPT-TCC bitfield Set */
//#define EDMA3CC_OPT_TCC_SET(tcc)    (((EDMA3CC_OPT_TCC >> EDMA3CC_OPT_TCC_SHIFT) & (tcc)) << EDMA3CC_OPT_TCC_SHIFT)
//                                    (    ( 0x0003F000u >>           0x0c )       &  tcc ) <<         0x0c )
// #define EDMA3CC_OPT_TCC (0x0003F000u)
// #define EDMA3CC_OPT_TCC_SHIFT (0x0000000Cu)
// #define EDMA3CC_OPT_TCC_SHIFT (0x0000000Cu)
#define PAR_RX_START                          (PAR_ID_START)  // 70u, Specify where the parameter set starting is
//--- Definitions which are not configurable -----------------
#define SIZE_PARAMSET                         (32u)
#define OPT_FIFO_WIDTH                        (0x02 << 8u)  // d.h. 32bit FIFO Width
//static
unsigned char rxBuf0[AUDIO_BUF_SIZE];
//static
unsigned char rxBuf1[AUDIO_BUF_SIZE];
//static
unsigned char rxBuf2[AUDIO_BUF_SIZE];

// Next buffer to receive data. The data will be received in this buffer.
//static volatile unsigned int nxtBufToRcv = 0;

// The RX buffer which filled latest.
//static volatile unsigned int lastFullRxBuf = 0;

// The offset of the paRAM ID, from the starting of the paRAM set.
//static volatile unsigned short parOffRcvd = 0;

//unsigned short parToSend;
//unsigned short parToLink;
//******************************************************************************
// nur Test
//******************************************************************************
void InitTest(void) {
	int var1;

	volatile unsigned int dw0, dw1, dw2, dw3, dw4, dw5, dw6;
	dw0 = NUM_RX_SERIALIZERS;// ((NUM_I2S_CHANNELS >> 1) + (NUM_I2S_CHANNELS & 0x01))
	dw1 = I2S_SLOTS;			// ((1 << NUM_I2S_CHANNELS) - 1)
	dw2 = BYTES_PER_SAMPLE;		// ((WORD_SIZE >> 3) * NUM_I2S_CHANNELS)
	dw3 = AUDIO_BUF_SIZE; // (NUM_SAMPLES_PER_AUDIO_BUF * BYTES_PER_SAMPLE    * NUM_RX_SERIALIZERS)
	dw4 = NUM_I2S_CHANNELS;
	dw5 = NUM_SAMPLES_PER_AUDIO_BUF;

#ifdef ENABLE_EDMA3
	dw6 = TRUE;
#else
	dw6 = FALSE;
#endif

	for (var1 = 0; var1 < AUDIO_BUF_SIZE; var1++) {
		rxBuf0[var1] = var1;
	}
	for (var1 = 0; var1 < AUDIO_BUF_SIZE; var1++) {
		rxBuf1[var1] = var1;
	}
}
/******************************************************************************
 **                      INTERNAL CONSTATNT DEFINITIONS
 ******************************************************************************/
/* Array of receive buffer pointers */
//static
unsigned int const rxBufPtr[NUM_BUF] = { (unsigned int) rxBuf0,   //,
		(unsigned int) rxBuf1, (unsigned int) rxBuf2 };

//--- Default paRAM for Receive section. --------------------
static struct EDMA3CCPaRAMEntry const rxDefaultPar = {
		(unsigned int) (OPT_FIFO_WIDTH), /* Opt field, 32bit breit */
		(unsigned int) SOC_MCASP_0_DATA_REGS, /* source address, auf MCASP_0 gendert */
		(unsigned short) (BYTES_PER_SAMPLE), /* aCnt */
		(unsigned short) (1), /* bCnt */
		(unsigned int) rxBuf0, /* dest address */
		(short) (0), /* source bIdx */
		(short) (BYTES_PER_SAMPLE), /* dest bIdx */
		(unsigned short) (PAR_RX_START * SIZE_PARAMSET), /* link address */
		(unsigned short) (0), /* bCnt reload value */
		(short) (0), /* source cIdx */
		(short) (0), /* dest cIdx */
		(unsigned short) 1 /* cCnt */
};

/*********************************************************************************
 * \brief   Sets the I2S format in the Transmit Format unit.
 *
 * \param   baseAddr      Base Address of the McASP Module Registers.
 * \param   wordSize      The word size in bits.
 * \param   slotSize      Slot size in bits.
 * \param   txMode        The mode of Transmission.
 *             txMode can take one of the values \n
 *                  MCASP_TX_MODE_NON_DMA - transmission without using DMA \n
 *                  MCASP_TX_MODE_DMA - DMA is used for transmission
 *
 * \return  None.
 *
 *  Note : The Frame Sync shall be separately configured for I2S.
 *         It is assumed that the word size is a multiple of 8.
 *
 *********************************************************************************/
void McASPTxFmtI2SSet_in(unsigned int baseAddr, unsigned int wordSize,
		unsigned int slotSize, unsigned int txMode) {
	/* Set the mask assuming integer format */
	McASPTxFmtMaskSet(baseAddr, (1 << wordSize) - 1);

	/* Set the transmit format unit for I2S */
	McASPTxFmtSet(baseAddr,
			(
			MCASP_TX_PAD_WITH_0 | MCASP_TX_BITSTREAM_MSB_FIRST
					| MCASP_TX_SYNC_DELAY_1BIT
					| ((wordSize >> 2) & MCASP_XFMT_XROT)
					| ((slotSize / 2 - 1) << MCASP_XFMT_XSSZ_SHIFT) | txMode));
}

/*********************************************************************************
 * \brief   Sets the I2S format in the Receive Format unit.
 *
 * \param   baseAddr      Base Address of the McASP Module Registers.
 * \param   wordSize      The word size in bits.
 * \param   slotSize      Slot size in bits.
 * \param   rxMode        The mode of reception.
 *             rxMode can take one of the values \n
 *                  MCASP_RX_MODE_NON_DMA - reception without using DMA \n
 *                  MCASP_RX_MODE_DMA - DMA is used for reception
 *
 * \return  None.
 *
 *  Note : The Frame Sync shall be separately configured for I2S.
 *         It is assumed that the word size is a multiple of 8.
 *
 *Abwandlung: 	8 Bit nach rechts rotieren (passiert automatisch durch wordSize=24 (= WORD_SIZE_MASK)
 *			 	auffllen mit Bit23 (RMASK wird hier durch wordSize=24 automatisch auf 0x00FF FFFF gesetzt)
 *********************************************************************************/
void McASPRxFmtI2SSet_in(unsigned int baseAddr, unsigned int wordSize,
		unsigned int slotSize, unsigned int rxMode) {
	volatile DWORD dw;
	/* Set the mask assuming integer format */
	// vorher:  McASPRxFmtMaskSet(baseAddr, (1 << wordSize) - 1);
	McASPRxFmtMaskSet(baseAddr, (1 << wordSize) - 1);

	// Set the receive format unit for I2S
	// vorher: McASPRxFmtSet(baseAddr, (MCASP_RX_PAD_WITH_0 | MCASP_RX_BITSTREAM_MSB_FIRST
	//                         | MCASP_RX_SYNC_DELAY_1BIT
	//                         | (((slotSize - wordSize) >> 2) & MCASP_RFMT_RROT)
	//                         | ((slotSize/2 -1) << MCASP_RFMT_RSSZ_SHIFT)
	//                         | rxMode));

	//dw = (((slotSize - wordSize) >> 2) & MCASP_RFMT_RROT); /////
	McASPRxFmtSet(baseAddr,
			(MCASP_RX_PAD_WITH_PBIT(23) //  vorher: MCASP_RX_PAD_WITH_0
			| MCASP_RX_BITSTREAM_MSB_FIRST | MCASP_RX_SYNC_DELAY_1BIT
					| (((slotSize - wordSize) >> 2) & MCASP_RFMT_RROT) // 2 --> 8 Bit nach rechts rotieren
					| ((slotSize / 2 - 1) << MCASP_RFMT_RSSZ_SHIFT)	// 0x0f auf Bit 7..4 --> Slot size ist 32bit
					| rxMode));

}

/***********************************************************************
 * \brief   Sets McASP pins to use for McASP functionality
 *			Siehe "22.4.1.3 PFUNC Register"
 * \param   baseAddr      Base Address of the McASP Module Registers.
 * \param   pinMask       Mask of the pins to be configured for McASP.
 *            pinMask can be a combination of the below values \n
 *                MCASP_PIN_AFSR  \n     BIT 31
 *                MCASP_PIN_AHCLKR \n    Bit 30
 *                MCASP_PIN_ACLKR \n     Bit 29
 *                MCASP_PIN_AFSX \n      Bit 28
 *                MCASP_PIN_AHCLKX \n    Bit 27
 *                MCASP_PIN_ACLKX \n     Bit 26
 *                MCASP_PIN_AMUTE \n     Bit 25
 *                MCASP_PIN_AXR(n) - For serializer 'n'     Bits 3...0
 *
 * \return  None.
 *
 **********************************************************************/
void McASPPinMcASPSet_in(unsigned int baseAddr, unsigned int pinMask) {
	//volatile unsigned int dw, dw1, dw2, dw3;
	//dw1 = (~pinMask);
	//dw2 = ~(0x1FFFFF << 4);
	//dw3 = dw1 & dw2;
	//dw = (~pinMask) & (~(0x1FFFFF << 4));   // Bits 24...4 mssen 0 sein (reserviert)

	//0x0 = Pin functions as McASP pin.   0x1 = Pin functions as GPIO pin.  S. 4356
	HWREG(baseAddr + MCASP_PFUNC) = (~pinMask) & (~(0x1FFFF << 4)); // Bits 24...4 mssen 0 sein (reserviert)
}
/****************************************************************************************
 * \brief   Configures the clock for the Receive Section for receiving bits.
 *          The source and divide values shall be pre-determined.
 *
 * \param   baseAddr      Base Address of the McASP Module Registers.
 * \param   clkSrc        The source of the clock
 * \param   mixClkDiv     Value which divides the mixed clock
 * \param   auxClkDiv     Value which divides the internal high frequency clock
 *
 *    clkSrc can take the values \n
 *         MCASP_RX_CLK_INTERNAL \n
 *         MCASP_RX_CLK_EXTERNAL \n
 *         MCASP_RX_CLK_MIXED \n
 *
 * \return  None.
 *
 * Note: If external clock is selected, the divide values will not be used. \n
 *       If mixed clock is selected, the clock divide mixClkDiv only
 *       will be used. \n
 *       If internal clock is selected, both the divide values will be used.
 *
 ****************************************************************************************/
/*
 void McASPRxClkCfg_in(unsigned int baseAddr, unsigned int clkSrc,
 unsigned int mixClkDiv, unsigned int auxClkDiv)
 {	volatile unsigned int dw0, dw1, dw2, dw3, dw4;



 dw0 = (MCASP_ACLKRCTL_CLKRM | MCASP_ACLKRCTL_CLKRDIV);
 dw1 = ~(MCASP_ACLKRCTL_CLKRM | MCASP_ACLKRCTL_CLKRDIV);

 // Bit7 = 1 --> Receiver tastet bei pos. Flanke ab -->ADC mu mit neg. Flanke abtasten
 // Bit6 = 0 reserviert
 // Bit5 = 0x0 = Internal receive clock source from output of programmable bit clock divider.
 //              ACLKXCTL.ASYNC = 1 notwendig, sonst wirkungslos (ACLKXCTL.ASYNC = 0 bei Reset)
 // Bits 4..0    Teiler
 //              nur wirksam, wenn ACLKXCTL.ASYNC = 1 (default: ACLKXCTL.ASYNC=0)
 HWREG(baseAddr + MCASP_ACLKRCTL) = ~(MCASP_ACLKRCTL_CLKRM | MCASP_ACLKRCTL_CLKRDIV); // res. Bits knnen nur gelesen werden
 //                                          ~( 0x20u              0x1Fu) = 0xFFFF FFC0







 dw2 = (MCASP_AHCLKRCTL_HCLKRM | MCASP_AHCLKRCTL_HCLKRDIV);
 dw3 = ~(MCASP_AHCLKRCTL_HCLKRM | MCASP_AHCLKRCTL_HCLKRDIV);

 HWREG(baseAddr + MCASP_AHCLKRCTL) &= ~(MCASP_AHCLKRCTL_HCLKRM | MCASP_AHCLKRCTL_HCLKRDIV);
 //                 0x74                          0x8000u                   0x0FFFu

 // Set the clock source to chose internal/external with clkdiv
 HWREG(baseAddr + MCASP_ACLKRCTL) |= ((clkSrc & MCASP_ACLKRCTL_CLKRM) | mixClkDiv);
 //                                                   0x20u

 HWREG(baseAddr + MCASP_AHCLKRCTL) |= ((clkSrc & MCASP_AHCLKRCTL_HCLKRM) | auxClkDiv);
 //                                                    0x8000u

 }
 */

/***********************************************************
 ** Configures the McASP Transmit Section in I2S mode.
 ***********************************************************/
//static
void McASPI2SConfigure(void) {
	volatile unsigned int dw, dw1;
	//McASPTxReset(SOC_MCASP_1_CTRL_REGS);
	McASPRxReset(SOC_MCASP_0_CTRL_REGS); 		// rcksetzen

	//--- Enable the FIFOs for DMA transfer --------------------------------------
	//dw = AFIFO_RFIFOCTL_RNUMEVT_SHIFT;   /////// = 8
	//dw = MCASP_FIFO_RFIFOCTL;   /////////// 8
	//dw = 2 | ((1 * 2) << AFIFO_RFIFOCTL_RNUMEVT_SHIFT);  ////// 0x202
	//dw = NUM_RX_SERIALIZERS | ((2 * NUM_RX_SERIALIZERS)<< AFIFO_RFIFOCTL_RNUMEVT_SHIFT);   //////// 0x402

	//dw = NUM_RX_SERIALIZERS;   //////////////// 2

	// Wenn Read-FIFO mind. 4 32-Bit-Worte enthlt, dann wird ein "receive DMA event" fr host bzw. DMA ausgelst
	// Read-FIFO liest bei einem "receive DMA event" vom McASP 2 32-Bit-Worte ein
	// Siehe auch "Receive DMA Event Pacer"
	McASPReadFifoEnable(SOC_MCASP_0_FIFO_REGS, NUM_RX_SERIALIZERS, 2); // 2 Receive Serializers benutzen
																	   // 2 - The minimum number of words per serializer
																	   //      to be available in FIFO to issue a DMA event.
	//McASPWriteFifoEnable(SOC_MCASP_1_FIFO_REGS, 1, 1);

	//--- Set I2S format in the transmitter/receiver format units ---------------
	//dw = MCASP_RX_MODE_DMA;   =0
	//vorher: McASPRxFmtI2SSet(SOC_MCASP_0_CTRL_REGS, WORD_SIZE, SLOT_SIZE,
	//		MCASP_RX_MODE_DMA); // setzt: MCASP_RX_SYNC_DELAY_1BIT, MCASP_RX_PAD_WITH_0!!!!!!! u.a.
	// MCASP_RX_MOD_DMA = 0 --> Reads from XRBUF[n] originate on data port.
	// 							Reads from XRBUF[n] on configuration bus are ignored.

	//vorher: McASPRxFmtI2SSet_in(SOC_MCASP_0_CTRL_REGS, WORD_SIZE, SLOT_SIZE,
	//MCASP_RX_MODE_DMA); // setzt: MCASP_RX_SYNC_DELAY_1BIT, MCASP_RX_PAD_WITH_0!!!!!!! u.a.
	// MCASP_RX_MOD_DMA = 0 --> Reads from XRBUF[n] originate on data port.
	// 							Reads from XRBUF[n] on configuration bus are ignored.

	McASPRxFmtI2SSet_in(SOC_MCASP_0_CTRL_REGS, WORD_SIZE_MASK, SLOT_SIZE,
	MCASP_RX_MODE_DMA); // setzt: MCASP_RX_SYNC_DELAY_1BIT, MCASP_RX_PAD_WITH_0!!!!!!! u.a.
						// MCASP_RX_MOD_DMA = 0 --> Reads from XRBUF[n] originate on data port.
						// 							Reads from XRBUF[n] on configuration bus are ignored.

//    McASPTxFmtI2SSet(SOC_MCASP_1_CTRL_REGS, WORD_SIZE, SLOT_SIZE, MCASP_TX_MODE_DMA);

	/* Configure the frame sync. I2S shall work in TDM format with 2 slots */
	McASPRxFrameSyncCfg(SOC_MCASP_0_CTRL_REGS, 2, MCASP_RX_FS_WIDTH_WORD, // 2 fr 2 Slots bei I2S
			MCASP_RX_FS_EXT_BEGIN_ON_FALL_EDGE);
	//McASPTxFrameSyncCfg(SOC_MCASP_1_CTRL_REGS, 2, MCASP_TX_FS_WIDTH_WORD, MCASP_TX_FS_EXT_BEGIN_ON_FALL_EDGE);

	// configure the clock for receiver
	//dw = MCASP_ACLKRCTL_CLKRM;   ////////////// 0x20
	//dw1 = MCASP_ACLKRCTL_CLKRDIV; ////////////// 0x1F
	//dw = MCASP_RX_CLK_EXTERNAL;   //////////////    0
	//dw = MCASP_AHCLKRCTL_HCLKRM;  /////////////// 0x8000
	//dw1 = MCASP_AHCLKRCTL_HCLKRDIV; ///////////// 0x0fff
	McASPRxClkCfg(SOC_MCASP_0_CTRL_REGS, MCASP_RX_CLK_EXTERNAL, 0, 0); // ext. Takt von aclkr,  vorher: MCASP_1
																	   // legt auch ahclkr als extern fest mit Teiler=1
	McASPRxClkPolaritySet(SOC_MCASP_0_CTRL_REGS, MCASP_RX_CLK_POL_RIS_EDGE); // vorher: MCASP_1
	//dw = MCASP_RX_CLKCHCK_DIV32; //////   5
//ha	McASPRxClkCheckConfig(SOC_MCASP_0_CTRL_REGS, MCASP_RX_CLKCHCK_DIV32, 0x00, 0xFF);  // vorher: MCASP_1

	// configure the clock for transmitter
	//McASPTxClkCfg(SOC_MCASP_1_CTRL_REGS, MCASP_TX_CLK_EXTERNAL, 0, 0);
	//McASPTxClkPolaritySet(SOC_MCASP_1_CTRL_REGS, MCASP_TX_CLK_POL_FALL_EDGE);
	//McASPTxClkCheckConfig(SOC_MCASP_1_CTRL_REGS, MCASP_TX_CLKCHCK_DIV32, 0x00, 0xFF);

	// Enable synchronization of RX and TX sections
	//McASPTxRxClkSyncEnable(SOC_MCASP_1_CTRL_REGS);

	// Enable the transmitter/receiver slots. I2S uses 2 slots
	McASPRxTimeSlotSet(SOC_MCASP_0_CTRL_REGS, I2S_SLOTS); // ist 0x03 ok fr 2 I2S-Kanle?, vorher: MCASP_1

	//McASPTxTimeSlotSet(SOC_MCASP_1_CTRL_REGS, I2S_SLOTS);

	//** Set the serializers, Currently only one serializer is set as
	//** transmitter and one serializer as receiver.
	//vorher: McASPSerializerTxSet(SOC_MCASP_1_CTRL_REGS, MCASP_XSER_TX);
	McASPSerializerRxSet(SOC_MCASP_0_CTRL_REGS, MCASP_XSER_RX_SDOUT1); // Serializer 0 als Empf. setzen, Eingang0, in mcasp.c,
	McASPSerializerRxSet(SOC_MCASP_0_CTRL_REGS, MCASP_XSER_RX_SDOUT2); // Serializer 1 als Empf. setzen, Eingang1, S. 4408    !!!!!!!!!!!!

	// Configure the McASP pins
	// Input - Frame Sync, Clock and Serializer Rx
	// (vorher: Output - Serializer Tx is connected to the input of the codec

	//vorher: McASPPinMcASPSet(SOC_MCASP_1_CTRL_REGS, 0xFFFFFFFF);
	McASPPinMcASPSet_in(SOC_MCASP_0_CTRL_REGS,    		// FSR, ACLKR, nur 2xAXR
			MCASP_PIN_AFSR | MCASP_PIN_ACLKR
			// | MCASP_PIN_AMUTE
			//| MCASP_PIN_AXR(3) | MCASP_PIN_AXR(2)
					| MCASP_PIN_AXR(1) | MCASP_PIN_AXR(0));

	// vorher: McASPPinDirOutputSet(SOC_MCASP_1_CTRL_REGS, MCASP_PIN_AXR(MCASP_XSER_TX));
	//McASPPinDirOutputSet(SOC_MCASP_0_CTRL_REGS, MCASP_PIN_AFSR     // wenn Pins als Ausgnge gesetzt werden sollten
	//											|MCASP_PIN_ACLKR
	//											);  // PDIR-Register, Takte als Ausgang

//    vorher: McASPPinDirInputSet(SOC_MCASP_1_CTRL_REGS, MCASP_PIN_AFSX
//                                               | MCASP_PIN_ACLKX
//                                               | MCASP_PIN_AFSR
//                                               | MCASP_PIN_ACLKR
//                                               | MCASP_PIN_AXR(MCASP_XSER_RX));
	McASPPinDirInputSet(SOC_MCASP_0_CTRL_REGS,
	MCASP_PIN_AFSR | MCASP_PIN_ACLKR | MCASP_PIN_AXR(MCASP_XSER_RX_SDOUT1) // axr0 als Eingang (fr SDOUT1 von ADC)
			| MCASP_PIN_AXR(MCASP_XSER_RX_SDOUT2) // axr1 als Eingang (fr SDOUT2 von ADC)   !!!!!!
	);

}
/******************************************************************
 ** Initializes the DMA parameters.
 * The RX basic paRAM set(channel) is 0 and TX basic paRAM set (channel) is 1.
 **
 ** The RX paRAM set 0 will be initialized to receive data in the rx buffer 0.
 ** The transfer completion interrupt will not be enabled for paRAM set 0;
 ** paRAM set 0 will be linked to linked paRAM set starting (PAR_RX_START) of RX.
 ** and further reception only happens via linked paRAM set.
 ** For example, if the PAR_RX_START value is 40, and the number of paRAMS is 2,
 ** reception paRAM set linking will be initialized as 0-->40-->41-->40
 **
 ** The TX paRAM sets will be initialized to transmit from the loop buffer.
 ** The size of the loop buffer can be configured.
 ** The transfer completion interrupt will not be enabled for paRAM set 1;
 ** paRAM set 1 will be linked to linked paRAM set starting (PAR_TX_START) of TX.
 ** All other paRAM sets will be linked to itself.
 ** and further transmission only happens via linked paRAM set.
 ** For example, if the PAR_RX_START value is 42, and the number of paRAMS is 2,
 ** So transmission paRAM set linking will be initialized as 1-->42-->42, 43->43.
 *********************************************************************/
//static
void I2SDMAParamInit(void) {
	volatile unsigned long dw;
	EDMA3CCPaRAMEntry paramSet;
	int idx;

	//#define BYTES_PER_SAMPLE                      ((WORD_SIZE >> 3) * NUM_I2S_CHANNELS)
	//volatile unsigned int dw0, dw1, dw2;
	//dw0 = WORD_SIZE;
	//dw1 = NUM_I2S_CHANNELS;
	//dw2 = BYTES_PER_SAMPLE;

	// Initialize the 0th paRAM set for receive
	memcpy(&paramSet, &rxDefaultPar, SIZE_PARAMSET - 2); // die letzten beiden Bytes sind reserviert, mssen 0 sein.

	EDMA3SetPaRAM(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP0_RX, &paramSet); // parRAM9 empfngt 1*BYTES_PER_SAMPLE Bytes (= bCnt*aCnt)
	//                   0x4900 0000           9                        // und schreibt sie nach rxBuf0

	// further paramsets, enable interrupt
	// dw = RX_DMA_INT_ENABLE;    //////////////// 0x10 9000
	paramSet.opt |= RX_DMA_INT_ENABLE; // 0x0010 9000 --> TCINTEN = 1,    TCC=9-->setzt chaining enable register CER[TCC]
									   //   transfer complete
	for (idx = 0; idx < NUM_PAR; idx++) {   // fr die folgenden ParRAMs
		paramSet.destAddr = rxBufPtr[idx];  // die richtige Zieladr. eintragen
		paramSet.linkAddr = (PAR_RX_START + ((idx + 1) % NUM_PAR)) // verlinken auf den nchsten parRAM
		* (SIZE_PARAMSET); // (70 + (0+1)%3) * 32 = Offset auf Num. f. nchsten PaRAM
						   // also 71+32, 72*32, 73*32
		paramSet.bCnt = NUM_SAMPLES_PER_AUDIO_BUF;

		// for the first linked paRAM set, start receiving the second
		// sample only since the first sample is already received in
		// rx buffer 0 itself. (Event 9 hat einmal BYTES_PER_SAMPLE Bytes an rxBuf0[] bertragen.)

		if (0 == idx) {
			paramSet.destAddr += BYTES_PER_SAMPLE;  // (Event 9 hat schon einmal
			// vorherparamSet.bCnt -= BYTES_PER_SAMPLE;		// BYTES_PER_SAMPLE Bytes an rxBuf0[] bertragen.)
			paramSet.bCnt -= 1;		// 1 Sample weniger an rxBuf0[] bertragen.)
		}

		//--- hinzugefgt: ----------------------------------
		if ((NUM_PAR - 1) == idx) {
			paramSet.linkAddr = 0xFFFF;	// Der letzte ParRAM soll transfer beenden

		}
		//---------------------------------------------------
		EDMA3SetPaRAM(SOC_EDMA30CC_0_REGS, (PAR_RX_START + idx), &paramSet);
	}
	// Initialize the required variables for reception
	//nxtBufToRcv = idx % NUM_BUF;
	//lastFullRxBuf = NUM_BUF - 1;
	//parOffRcvd = 0;

	/* Initialize the 1st paRAM set for transmit */
//    memcpy(&paramSet, &txDefaultPar, SIZE_PARAMSET - 2);
//    EDMA3SetPaRAM(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP1_TX, &paramSet);
	/* rest of the params, enable loop job */
//    for(idx = 0 ; idx < NUM_PAR; idx++)     {
//        ParamTxLoopJobSet(PAR_TX_START + idx);
//    }
	/* Initialize the variables for transmit */
//    parOffSent = 0;
//    lastSentTxBuf = NUM_BUF - 1;
}

/*
 ** Activates the DMA transfer for a parameter set from the given buffer.
 */
#ifdef MCASP
static void BufferRxDMAActivate(unsigned int rxBuf, unsigned short parId,
		unsigned short parLink) {

	EDMA3CCPaRAMEntry paramSet;

	/* Copy the default paramset */
	memcpy(&paramSet, &rxDefaultPar, SIZE_PARAMSET - 2);

	/* Enable completion interrupt */
	paramSet.opt |= RX_DMA_INT_ENABLE;
	paramSet.destAddr = rxBufPtr[rxBuf];
	paramSet.bCnt = NUM_SAMPLES_PER_AUDIO_BUF;
	paramSet.linkAddr = parLink * SIZE_PARAMSET;

	EDMA3SetPaRAM(SOC_EDMA30CC_0_REGS, parId, &paramSet);
}
#endif

/****************************************************************************
 ** This function will be called once receive DMA is completed
 ****************************************************************************/
static void McASPRxDMAComplHandler_in(void) {
	volatile unsigned int dw, dw0, Max;
	dw = 0x12345678;

#ifdef MAKE_EXCEL_FILE
	dw0 = NUM_BUF * NUM_SAMPLES_PER_AUDIO_BUF * BYTES_PER_SAMPLE/4; //////////////////////////
	Max = GetMax((int *)rxBuf0, NUM_BUF * NUM_SAMPLES_PER_AUDIO_BUF * BYTES_PER_SAMPLE/4);
	//MkAsciiTestFile((int *)rxBuf0, NUM_BUF*NUM_SAMPLES_PER_AUDIO_BUF, NUM_I2S_CHANNELS, Max);
	MkAsciiTestFile((int *)rxBuf0, 192, 4, Max);

#endif
	//unsigned short nxtParToUpdate;
	//Z2++;  /////////////////////////
	// Update lastFullRxBuf to indicate a new buffer reception is completed.
	//lastFullRxBuf = (lastFullRxBuf + 1) % NUM_BUF;
	//nxtParToUpdate = PAR_RX_START + parOffRcvd;
	//parOffRcvd = (parOffRcvd + 1) % NUM_PAR;

	// Update the DMA parameters for the received buffer to receive
	// further data in proper buffer
	//BufferRxDMAActivate(nxtBufToRcv, nxtParToUpdate, PAR_RX_START + parOffRcvd);

	// update the next buffer to receive data
	//nxtBufToRcv = (nxtBufToRcv + 1) % NUM_BUF;
}

/*************************************************************
 ** ISR for McASP RX section
 *************************************************************/
/* RINTCTL


 #define MCASP_RINTCTL_RSTAFRM (0x00000080u)
 #define MCASP_RINTCTL_RSTAFRM_SHIFT (0x00000007u)

 #define MCASP_RINTCTL_RDATA (0x00000020u)
 #define MCASP_RINTCTL_RDATA_SHIFT (0x00000005u)

 #define MCASP_RINTCTL_RLAST (0x00000010u)
 #define MCASP_RINTCTL_RLAST_SHIFT (0x00000004u)

 #define MCASP_RINTCTL_RDMAERR (0x00000008u)
 #define MCASP_RINTCTL_RDMAERR_SHIFT (0x00000003u)

 #define MCASP_RINTCTL_RCKFAIL (0x00000004u)
 #define MCASP_RINTCTL_RCKFAIL_SHIFT (0x00000002u)

 #define MCASP_RINTCTL_RSYNCERR (0x00000002u)
 #define MCASP_RINTCTL_RSYNCERR_SHIFT (0x00000001u)

 #define MCASP_RINTCTL_ROVRN (0x00000001u)
 #define MCASP_RINTCTL_ROVRN_SHIFT (0x00000000u)
 */

static void McASPRxIsr(void) {
	// #define MCASP_RBUF(n)			(0x280 + (n * 4))
	volatile unsigned int dw, dw1, dw3, dw4, dw5;

	; /* Perform any error handling here.*/

	dw1 = McASPRxStatusGet(SOC_MCASP_0_CTRL_REGS);
	dw = HWREG(SOC_MCASP_0_CTRL_REGS + MCASP_RSTAT);  // Statusregister lesen
	// 0x104   Bit8 -->receiver error interrupt
	//         Bit2 -->Receive clock failure flag.
	//--- Daten lesen, wenn vorhanden -------------------
	if (dw & MCASP_RSTAT_RDATA) {
		dw4 = HWREG(SOC_MCASP_0_CTRL_REGS + MCASP_RBUF(0));  // RBUF(0) lesen
		//dw5 = HWREG(SOC_MCASP_0_CTRL_REGS + MCASP_RBUF(1));  // RBUF(1) lesen
	}

	//--- Statusbits lschen in RSTAT--------------Def. in hw_mcasp.h
	HWREG(SOC_MCASP_0_CTRL_REGS + MCASP_RSTAT) =
	// Bit8 -->receiver error interrupt,  nicht schreiben
			MCASP_RSTAT_RDMAERR			// Bit7
			| MCASP_RSTAT_RSTAFRM     	// Bit6
					| MCASP_RSTAT_RDATA			// Bit5
					| MCASP_RSTAT_RLAST			// Bit4
												// Bit3, nicht schreiben
					| MCASP_RSTAT_RCKFAIL// Bit2 -->Receive clock failure flag.
					| MCASP_RSTAT_RSYNCERR		// Bit1
					| MCASP_RSTAT_ROVRN;		// Bit0

	dw3 = 0x12345678;
}

/*************************************************************
 ** Sets up the interrupts for McASP in AINTC
 **************************************************************/
//static
void McASPIntSetup(void) {
	//IntRegister(SYS_INT_MCATXINT1, McASPTxIsr);
	IntRegister(SYS_INT_MCARXINT0, McASPRxIsr); // McASP0: ISR "McASPRxIsr()" auf Pos. 81 in Tab.,  vorher: SYS_INT_MCARXINT1
	//                81
	//IntPrioritySet(SYS_INT_MCATXINT1, 0, AINTC_HOSTINT_ROUTE_IRQ);
	IntPrioritySet(SYS_INT_MCARXINT0, 0, AINTC_HOSTINT_ROUTE_IRQ);    // Prio. 0

	/* Enabling the system interrupt in AINTC. */
	//IntSystemEnable(SYS_INT_MCATXINT1);
	IntSystemEnable(SYS_INT_MCARXINT0);

}
/**********************************************************************
 ** Sets up the interrupts for EDMA in AINTC
 **********************************************************************/
// aus mcasp.c
//static
void EDMA3IntSetup_in(void) {
	IntRegister(SYS_INT_EDMACOMPINT, EDMA3CCComplIsr_in); // ISR auf Pos. 12 schreiben
	//              12
	IntPrioritySet(SYS_INT_EDMACOMPINT, 0, AINTC_HOSTINT_ROUTE_IRQ); // oberste Prioritt
	//                  12                        0
	/* Enabling the system interrupt in AINTC. */
	IntSystemEnable(SYS_INT_EDMACOMPINT);  // 12
}
/*******************************************************************
 * \brief   This function enables McASP clocks
 *          und wartet bis volle Funktionalitt erreicht ist.
 * \param   None
 *
 * \return  None.
 *
 ******************************************************************/
void McASP0ModuleClkConfig(void) {
	//HWREG(SOC_PRCM_REGS + CM_PER_MCASP1_CLKCTRL) = CM_PER_MCASP1_CLKCTRL_MODULEMODE_ENABLE;
	//      0x44E0 0000             0x68                      0x2u

	HWREG(SOC_PRCM_REGS + CM_PER_MCASP0_CLKCTRL) =
	CM_PER_MCASP0_CLKCTRL_MODULEMODE_ENABLE; // 2 - Enable module
	//      0x44E0 0000             0x34                      0x2u

	while ((HWREG(SOC_PRCM_REGS + CM_PER_MCASP0_CLKCTRL)
			& CM_PER_MCASP0_CLKCTRL_MODULEMODE)
			!= CM_PER_MCASP0_CLKCTRL_MODULEMODE_ENABLE)
		;

	/*
	 ** Waiting for IDLEST field in CM_PER_MCASP1_CLKCTRL register to attain the
	 ** desired value. Warten bis volle Funktionalitt erreicht ist.
	 */
	while ((CM_PER_MCASP0_CLKCTRL_IDLEST_FUNC
			<< CM_PER_MCASP0_CLKCTRL_IDLEST_SHIFT)
			!= (HWREG(SOC_CM_PER_REGS + CM_PER_MCASP0_CLKCTRL)
					& CM_PER_MCASP0_CLKCTRL_IDLEST))
		;

}

/*******************************************************************
 * \brief   InitMcAsp0()
 * \param   None
 * \return  None.
 ******************************************************************/
void InitMcAsp0(void) {
	//--- enable ------------------------------------------------
	McASP0ModuleClkConfig();   	// enable McASP0
	//--- Test ---------------
	FillRxBufs(); 				// alle Empfangspuffer vorbelegen (Test)

	//---
#ifdef ENABLE_EDMA3
	EDMAModuleClkConfig();  // in ...\platform\beaglebone\edma.c,
							// function enables the system L3 clocks and clocks for EDMA instance
#endif

	//IntAINTCInit();       // in ...\StarterWare...\system_config\arm7a\arm335x\interrupt.c
	//Initialize the ARM Interrupt Controller. "IntDefaultHandler()" fr alle

	// Initialize the I2C 0 interface for the codec AIC31
	//I2CCodecIfInit(SOC_I2C_1_REGS, I2C_SLAVE_CODEC_AIC31);  nicht bentigt

#ifdef ENABLE_EDMA3
	EDMA3Init_in(SOC_EDMA30CC_0_REGS, 0);  // --> regionId = 0
	EDMA3IntSetup_in();  // interrupts for EDMA in AINTC
						 // EDMA3CCComplIsr_in() noch anpassen !!!!!!!!!!!!!!!! EDMA transfer completion ISR
#endif

	McASPIntSetup();  	// ISR "McASPRxIsr()" auf Pos. 81 in Tab. setzen
						// Fehlerbehandlung, noch schreiben, interrupts for McASP in AINTC

	// Enable the interrupts generation at global level
	IntMasterIRQEnable();  		// IRQ enablen (CPSR[I]=0

	/* Request EDMA channels */
#ifdef ENABLE_EDMA3
	EDMA3RequestChannel(SOC_EDMA30CC_0_REGS, EDMA3_CHANNEL_TYPE_DMA, // 0x4900 0000, 0
			EDMA3_CHA_MCASP0_RX, EDMA3_CHA_MCASP0_RX, 0); // 09, 09, 0
	//=09u, in edma_event_in.h definiert, siehe S. 1302 (vorher: EDMA3_CHA_MCASP1_RX)

	// Initialize the DMA parameters
	I2SDMAParamInit();   // anpassen !!!!!!!!!!!!!!!!!!!!
#endif

	/* Configure the Codec for I2S mode */
	//AIC31I2SConfigure();   nicht bentigt

	//--- eingefgt, -------------------------------------
	McASPTxRxClkSyncDisable(SOC_MCASP_0_CTRL_REGS); // Takte von RX u. TX nicht synchron, sonst kommt externer ACLKR nicht durch

	//---Interrupts erlauben f. RINTCTL ------------------
	// Test: McASPRxIsr() auslsen, wenn Daten empfangen bzw. wenn Fehler
	HWREG(SOC_MCASP_0_CTRL_REGS + MCASP_RINTCTL) = //Register RINTCTL, Definitionen in hw_mcasp.h
			//                               0x7C
			0
			//| MCASP_RINTCTL_RSTAFRM		// Bit7 receive start of frame
			// Bit6 reserved
			//| MCASP_RINTCTL_RDATA		// Bit5  Interrupt is enabled. A receive data ready interrupt
			//       generates a McASP receive interrupt (RINT).
			//| MCASP_RINTCTL_RLAST		// Bit4  receive last slot
			//| MCASP_RINTCTL_RDMAERR		// Bit3  receive DMA error interrupt
			//| MCASP_RINTCTL_RCKFAIL   // Bit2, erst enablen nach spezieller Prozedur, s.S. 4337
			//| MCASP_RINTCTL_RSYNCERR	// Bit1
			//| MCASP_RINTCTL_ROVRN      // Bit0
			;

	//--- Configure the McASP for I2S -------------------
	McASPI2SConfigure();        //   anpassen!!!!!!!!!!!!!!
								// Achtung! ClkCheck deaktiviert!!!!!!!!!!
	//---------------------------------------------------
	// Sptestens hier mssen die externen Takte zugeschaltet sein, sonst lassen sich die
	// Serializer nicht aktivieren in McASPRxSerActivate()(PinMux: Clocks als Eingnge!!).
	//---------------------------------------------------

}

//***************************************************************
// Fkt.:	InitMcASP0Pins()
//          Pins fr McASP0 multiplexen
//***************************************************************
void InitMcASP0Pins(void) {

	//--- HDMI-Oscillator sperren --------------------------------
	// wird in InitGpio1() ber GPIO1_27=LOW gesperrt,
	// wahrscheinlich nicht ntig, da Oszillator bei Start (ohne Sperrung) auch nicht luft

	//--- enable ------------------------------------------------
	McASP0ModuleClkConfig();   	// enable McASP0

	//--- Pin Multiplexing ---------------------------------------
	// sind parallel geschaltet:
	MUX_VAL(CONTROL_PADCONF_XDMA_EVENT_INTR1, (IEN | PU | MODE7 ));
	// pin41: gpio0[20] als Eingang, ist mit Mcasp0_axr1 verbunden, sonst abklemmen
	MUX_VAL(CONTROL_PADCONF_ECAP0_IN_PWM0_OUT, (IEN | PU | MODE7 ));
	// pin42: gpio0[7] als Eingang, ist mit Mcasp0_axr2 verbunden, sonst abklemmen

	// McASP0-Pins:
	MUX_VAL(CONTROL_PADCONF_MCASP0_AXR0, (IEN | PD | MODE0 ));
	// mcasp0_axr0_mux0, Kanal 0 u. 1

	MUX_VAL(CONTROL_PADCONF_MCASP0_AXR1, (IEN | PD | MODE0 ));
	// mcasp0_axr1_mux0, Kanal 2 u. 3

	MUX_VAL(CONTROL_PADCONF_MCASP0_AHCLKR, (IEN | PD | MODE2 ));
	// mcasp0_axr2_mux0, Reserve

	MUX_VAL(CONTROL_PADCONF_MCASP0_AHCLKX, (IEN | PD | MODE2 ));
	// mcasp0_axr3_mux0, Reserve

	MUX_VAL(CONTROL_PADCONF_MCASP0_FSR, (IEN | PD | MODE0 ));
	// mcasp0_fsr_mux0

	MUX_VAL(CONTROL_PADCONF_MCASP0_ACLKR, (IEN | PD | MODE0 ));
	// mcasp0_aclkr_mux0

	//   if(FALSE == McASP1PinMuxSetup())
	// {
	// ConsoleUtilsPrintf("\n\rNot able to do pin multiplexing. ");
	// ConsoleUtilsPrintf("Please verify the profile setting.");
	// }
}
/**************************************************************************
 * \brief   Enables the McASP Reception. When this API is called,
 *          The receive state machine and the frame sync generators are
 *          released from reset. The McASP starts reception on the
 *          first frame sync after this.
 *
 * \param   baseAddr      Base Address of the McASP Module Registers.
 *
 * \return  None.
 *
 *************************************************************************/
void McASPRxEnable_in(unsigned int baseAddr) {
	//ConsoleUtilsPrintf("before: regionId = %x\r\n", regionId),

	// Release the Receive State machine from reset
	HWREG(baseAddr + MCASP_GBLCTL) |= MCASP_GBLCTL_RSMRST;
	//    0x4803 8000    0x44             0x08u
	//ConsoleUtilsPrintf("after: regionId = %x\r\n", regionId);

	while ((HWREG(baseAddr + MCASP_GBLCTL) & MCASP_GBLCTL_RSMRST)
			!= MCASP_GBLCTL_RSMRST)
		;

	//Release the frame sync generator from reset, lt. Doku auch freigeben, wenn FrameSync extern erzeugt wird
	HWREG(baseAddr + MCASP_GBLCTL) |= MCASP_GBLCTL_RFRST;
	while ((HWREG(baseAddr + MCASP_GBLCTL) & MCASP_GBLCTL_RFRST)
			!= MCASP_GBLCTL_RFRST)
		;

}

/*
 void McASPRxEnable(unsigned int baseAddr)
 {
 // Release the Receive State machine from reset
 HWREG(baseAddr + MCASP_GBLCTL) |= MCASP_GBLCTL_RSMRST;
 while((HWREG(baseAddr + MCASP_GBLCTL) & MCASP_GBLCTL_RSMRST)
 != MCASP_GBLCTL_RSMRST) ;

 // Release the frame sync generator from reset
 HWREG(baseAddr + MCASP_GBLCTL) |= MCASP_GBLCTL_RFRST;
 while((HWREG(baseAddr + MCASP_GBLCTL) & MCASP_GBLCTL_RFRST)
 != MCASP_GBLCTL_RFRST) ;
 }

 */
/****************************************************************
 ** Activates the data transmission/reception
 ** The DMA parameters shall be ready before calling this function.
 ****************************************************************/
//static
void I2SDataTxRxActivate_in(void) {
	/* Start the clocks */
	McASPRxClkStart(SOC_MCASP_0_CTRL_REGS, MCASP_RX_CLK_EXTERNAL);
	//McASPTxClkStart(SOC_MCASP_1_CTRL_REGS, MCASP_TX_CLK_EXTERNAL);

	/* Enable EDMA for the transfer */
#ifdef ENABLE_EDMA3
	EDMA3EnableTransfer(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP0_RX,
	EDMA3_TRIG_MODE_EVENT);
#endif
	//EDMA3EnableTransfer(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP1_TX,
	//                    EDMA3_TRIG_MODE_EVENT);

	/* Activate the  serializers */
	McASPRxSerActivate(SOC_MCASP_0_CTRL_REGS); // aktiviert  alle receive serializers ((externe) Takte mssen anliegen)
	//McASPTxSerActivate(SOC_MCASP_1_CTRL_REGS);

	// make sure that the XDATA bit is cleared to zero
	//while(McASPTxStatusGet(SOC_MCASP_0_CTRL_REGS) & MCASP_TX_STAT_DATAREADY);

	// Activate the state machines
	McASPRxEnable_in(SOC_MCASP_0_CTRL_REGS);
	//McASPTxEnable(SOC_MCASP_1_CTRL_REGS);
}

//*****************************************************************************
//Fkt.:	FillBuf()

//*****************************************************************************
void Fill(unsigned int adr, unsigned int len, char val) {
	unsigned int i;
	char *pt;
	pt = (char *) adr;
	for (i = 0; i < len; i++) {
		pt[i] = val;
	}

}

//*****************************************************************************
// Fkt.:	FillRxBufs()
//          Alle Empfangspuffer vorbelegen (Test)
//*****************************************************************************
void FillRxBufs(void) {
	int i;
	volatile unsigned int dw;
	dw = rxBufPtr[0];
	for (i = 0; i < NUM_BUF; i++) {
		Fill(rxBufPtr[i], AUDIO_BUF_SIZE, (char) i);
	}
}

//****************************************************************************
void Test(void) {
	volatile int Test;
	Test = 0x12345678;
	Test = 0x11223344;

}

//*************************************************************************
// Fkt.:	UpdateRxBufSettings()
//			Die Zeiger auf die einzelnen RxBuf[] aktualisieren nachdem Puffer voll
//          Aufruf in main()
//*************************************************************************
/*
 void UpdateRxBufSettings(void) {
 if (lastFullRxBuf != lastSentTxBuf) {

 // Start the transmission from the link paramset. The param set
 // 1 will be linked to param set at PAR_TX_START. So do not
 // update paRAM set1.

 parToSend = PAR_TX_START + (parOffTxToSend % NUM_PAR);
 parOffTxToSend = (parOffTxToSend + 1) % NUM_PAR;
 parToLink = PAR_TX_START + parOffTxToSend;

 lastSentTxBuf = (lastSentTxBuf + 1) % NUM_BUF;

 // Copy the buffer
 memcpy((void *) txBufPtr[lastSentTxBuf],
 (void *) rxBufPtr[lastFullRxBuf], AUDIO_BUF_SIZE);

 // Send the buffer by setting the DMA params accordingly.
 // Here the buffer to send and number of samples are passed as
 // parameters. This is important, if only transmit section
 // is to be used.

 //BufferTxDMAActivate(lastSentTxBuf, NUM_SAMPLES_PER_AUDIO_BUF,
 //     (unsigned short) parToSend, (unsigned short) parToLink);
 }
 }
 */

/***********************************************************************
 ** EDMA transfer completion ISR
 ************************************************************************/
//static
void EDMA3CCComplIsr_in(void) {
	volatile unsigned int dw0;
	//volatile unsigned int *pdw1;

	dw0 = regionId;   ////////////////
	//pdw1 = &regionId;  ////////////////

	// Check if receive DMA completed
	if (EDMA3GetIntrStatus(SOC_EDMA30CC_0_REGS) & (1 << EDMA3_CHA_MCASP0_RX)) // 9,  AREVT0 fr MCASP0, siehe "11.3.20 EDMA Events",  S. 1302
																			  // ist die Nummer f. das PaRAM-Element
			{
		// Clear the interrupt status for the 0th channel
		EDMA3ClrIntr(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP0_RX);
		McASPRxDMAComplHandler_in();
	}

	// Check if transmit DMA completed
	//if(EDMA3GetIntrStatus(SOC_EDMA30CC_0_REGS) & (1 << EDMA3_CHA_MCASP1_TX))
	//{
	// Clear the interrupt status for the first channel
	//EDMA3ClrIntr(SOC_EDMA30CC_0_REGS, EDMA3_CHA_MCASP1_TX);
	//McASPTxDMAComplHandler();
	//}
}

/*
 void DoMcAsp(void) {

 if(lastFullRxBuf != lastSentTxBuf)
 {

 // Start the transmission from the link paramset. The param set
 // 1 will be linked to param set at PAR_TX_START. So do not
 // update paRAM set1.

 //parToSend =  PAR_TX_START + (parOffTxToSend % NUM_PAR);
 parOffTxToSend = (parOffTxToSend + 1) % NUM_PAR;
 parToLink  = PAR_TX_START + parOffTxToSend;

 lastSentTxBuf = (lastSentTxBuf + 1) % NUM_BUF;

 // Copy the buffer
 memcpy((void *)txBufPtr[lastSentTxBuf],
 (void *)rxBufPtr[lastFullRxBuf],
 AUDIO_BUF_SIZE);


 // Send the buffer by setting the DMA params accordingly.
 // Here the buffer to send and number of samples are passed as
 // parameters. This is important, if only transmit section
 // is to be used.

 BufferTxDMAActivate(lastSentTxBuf, NUM_SAMPLES_PER_AUDIO_BUF,
 (unsigned short)parToSend,
 (unsigned short)parToLink);
 }
 }
 */

/****************************** End Of File *********************************/
